﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static Filsonator.MainForm;
using static System.Net.Mime.MediaTypeNames;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrayNotify;

namespace Filsonator
{
    public partial class StructureForm : Form
    {
        public StructureForm()
        {
            InitializeComponent();
        }

        private void StructureForm_Load(object sender, EventArgs e)
        {
            UpdateList();
        }

        public void UpdateList()
        {
            Settings sj = JsonConvert.DeserializeObject<Settings>(File.ReadAllText($@"settings.json"));
            var str = sj.Structures;

            dataGridView_Structures.Rows.Clear();
            dataGridView_Structures.RowTemplate.Height = 40;

            foreach (var s in str)
            {
                dataGridView_Structures.Rows.Add(s, Utilities.Draw.DrawStructure(Color.White, Color.FromArgb(69, 53, 40), Color.Black, 45, s));
            }
        }

        private Rectangle dragBoxFromMouseDown;
        private int rowIndexFromMouseDown;
        private int rowIndexOfItemUnderMouseToDrop;

        private void dataGrid_Main_Structures_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                // If the mouse moves outside the rectangle, start the drag.
                if (dragBoxFromMouseDown != Rectangle.Empty &&
                !dragBoxFromMouseDown.Contains(e.X, e.Y))
                {
                    // Proceed with the drag and drop, passing in the list item.                    
                    DragDropEffects dropEffect = dataGridView_Structures.DoDragDrop(
                          dataGridView_Structures.Rows[rowIndexFromMouseDown],
                          DragDropEffects.Move);
                }
            }
        }

        private void dataGrid_Main_Structures_MouseDown(object sender, MouseEventArgs e)
        {
            // Get the index of the item the mouse is below.
            rowIndexFromMouseDown = dataGridView_Structures.HitTest(e.X, e.Y).RowIndex;

            if (rowIndexFromMouseDown != -1)
            {
                // Remember the point where the mouse down occurred. 
                // The DragSize indicates the size that the mouse can move 
                // before a drag event should be started.                
                Size dragSize = SystemInformation.DragSize;

                // Create a rectangle using the DragSize, with the mouse position being
                // at the center of the rectangle.
                dragBoxFromMouseDown = new Rectangle(
                          new Point(
                            e.X - (dragSize.Width / 2),
                            e.Y - (dragSize.Height / 2)),
                      dragSize);
            }
            else
                // Reset the rectangle if the mouse is not over an item in the ListBox.
                dragBoxFromMouseDown = Rectangle.Empty;
        }

        private void dataGrid_Main_Structures_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }

        private void dataGrid_Main_Structures_DragDrop(object sender, DragEventArgs e)
        {
            // The mouse locations are relative to the screen, so they must be 
            // converted to client coordinates.
            Point clientPoint = dataGridView_Structures.PointToClient(new Point(e.X, e.Y));

            // Get the row index of the item the mouse is below. 
            rowIndexOfItemUnderMouseToDrop = dataGridView_Structures.HitTest(clientPoint.X, clientPoint.Y).RowIndex;

            // If the drag operation was a move then remove and insert the row.
            if (e.Effect == DragDropEffects.Move)
            {
                DataGridViewRow rowToMove = e.Data.GetData(typeof(DataGridViewRow)) as DataGridViewRow;
                dataGridView_Structures.Rows.RemoveAt(rowIndexFromMouseDown);
                dataGridView_Structures.Rows.Insert(rowIndexOfItemUnderMouseToDrop, rowToMove);

            }
        }
        public class Settings
        {
            public string league { get; set; }
            public List<string> Filters { get; set; }
            public List<string> Structures { get; set; }
            public int tierValue { get; set; }
            public bool tier1HAxe { get; set; }
            public bool tier2HAxe { get; set; }
            public bool tierBow { get; set; }
            public bool tierClaw { get; set; }
            public bool tierDagger { get; set; }
            public bool tier1HMace { get; set; }
            public bool tier2HMace { get; set; }
            public bool tierSceptre { get; set; }
            public bool tierStaff { get; set; }
            public bool tier1HSword { get; set; }
            public bool tier2HSword { get; set; }
            public bool tierWand { get; set; }
            public bool tierBody { get; set; }
            public bool tierBoots { get; set; }
            public bool tierGloves { get; set; }
            public bool tierHelmet { get; set; }
            public bool tierShield { get; set; }
            public bool tierQuiver { get; set; }
            public int uniqueValue { get; set; }
            public int fossilValue { get; set; }
            public int cardValue { get; set; }
            public List<string> tierBaseInclude { get; set; }
            public List<string> tierBaseExclude { get; set; }
            public bool install { get; set; }
            public string path { get; set; }
            public string structure { get; set; }
            public List<string> Sizes { get; set; }
            public List<string> Colors { get; set; }
            public List<string> Shapes { get; set; }
            public List<string> Actions { get; set; }
            public List<string> Conditions { get; set; }
            public List<string> Operators { get; set; }
        }

        private void button_Close_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void button_Save_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("Are you sure? The structures order is major in importance for in-game result. Priority in structure is reflected in the final item filter.", "Confirmation", MessageBoxButtons.YesNo);
            if (dr == DialogResult.Yes)
            {
                string json = File.ReadAllText($@"settings.json");
                dynamic jo = JsonConvert.DeserializeObject(json);
                JArray structures = new JArray();
                foreach (DataGridViewRow row in dataGridView_Structures.Rows)
                {
                    structures.Add(row.Cells[0].Value);
                    jo["Structures"] = structures;
                    string output = JsonConvert.SerializeObject(jo, Formatting.Indented);
                    File.WriteAllText($@"settings.json", output);                    
                }
                UpdateList();
            }
            else if (DialogResult == DialogResult.No)
            {
                //Nothing to do
            }
        }
    }
}
